package game

import (
	"fmt"
	"net"
	"time"
)

// Player represents a logged in client.
type Player struct {
	*Account
	game       *Game
	server     *Server
	authToken  string
	address    *net.UDPAddr
	direction  Vector3
	lastPacket time.Time
}

// NewPlayer creates a new player.
func NewPlayer(address *net.UDPAddr, account *Account, game *Game) *Player {
	return &Player{
		Account: account,
		address: address,
		game:    game,
		server:  game.server,
	}
}

// String shows the client address.
func (c *Player) String() string {
	return c.address.String()
}

// KeepAlive sets the last packet time to now.
func (c *Player) KeepAlive() {
	c.lastPacket = time.Now()
}

// Tick is run on every tick.
func (c *Player) Tick() {
	// ...
}

// State returns the player state (ID, name and position).
func (player *Player) State() []byte {
	state := []byte{}
	state = AppendString(state, player.ID)
	state = AppendString(state, player.Name)
	state = AppendVector3(state, player.Position)
	return state
}

// Send sends a message to the player.
func (player *Player) Send(code byte, data []byte) {
	player.server.Send(code, data, player.address)
}

// OnConnect is executed when a connection has been established.
func (player *Player) OnConnect() {
	fmt.Printf("%s connected.\n", player.Name)
	players := player.game.players

	players.Each(func(other *Player) {
		player.Send(PlayerAdd, other.State())

		if other != player {
			other.Send(PlayerAdd, player.State())
		}
	})
}

// OnDisconnect is executed when the connection has been closed.
func (player *Player) OnDisconnect() {
	fmt.Printf("%s disconnected.\n", player.Name)
	player.game.BroadcastOthers(PlayerRemove, []byte(player.ID), player)
}
