package game

import (
	"net"
	"time"
)

// PlayerManager keeps track of all players.
type PlayerManager struct {
	players  map[string]*Player
	accounts map[string]*Player
	count    int
}

// NewPlayerManager creates a new player manager.
func NewPlayerManager() *PlayerManager {
	return &PlayerManager{
		players:  map[string]*Player{},
		accounts: map[string]*Player{},
	}
}

// Add adds a new player.
func (m *PlayerManager) Add(player *Player) {
	m.players[player.address.String()] = player
	m.accounts[player.ID] = player
	m.count++
	player.OnConnect()
}

// ByAddress returns an existing client for the requested address.
func (m *PlayerManager) ByAddress(address *net.UDPAddr) *Player {
	player, exists := m.players[address.String()]

	if !exists {
		return nil
	}

	return player
}

// ByAccount returns the player with the given account ID.
func (m *PlayerManager) ByAccount(id string) *Player {
	player, exists := m.accounts[id]

	if !exists {
		return nil
	}

	return player
}

// ChangeAddress changes the address of a player.
func (m *PlayerManager) ChangeAddress(player *Player, address *net.UDPAddr) {
	delete(m.players, player.address.String())
	player.address = address
	m.players[player.address.String()] = player
}

// Clean checks for players who haven't responded in the timeout duration and disconnects them.
func (m *PlayerManager) Clean(timeout time.Duration) {
	now := time.Now()

	for _, player := range m.players {
		if player.lastPacket.IsZero() || now.Before(player.lastPacket.Add(timeout)) {
			continue
		}

		m.Remove(player)
	}
}

// Contains tells you whether the address is already a registered client.
func (m *PlayerManager) Contains(address *net.UDPAddr) bool {
	_, exists := m.players[address.String()]
	return exists
}

// Count returns the number of clients.
func (m *PlayerManager) Count() int {
	return m.count
}

// Each calls the callback function for each client.
func (m *PlayerManager) Each(callback func(*Player)) {
	for _, player := range m.players {
		callback(player)
	}
}

// Remove removes a player.
func (m *PlayerManager) Remove(player *Player) {
	delete(m.players, player.address.String())
	delete(m.accounts, player.ID)
	m.count--
	player.OnDisconnect()
}
